#ifndef _SNMPPD_H_
#define _SNMPPD_H_ 1

/* Defines */
#ifndef FALSE
# define FALSE 0
#endif
#ifndef TRUE
# define TRUE !FALSE
#endif

/* Some useful defaults */
#define NUM_WORKERS 15
#define MAX_SNMPPD_OID_LEN 1024
#define SNMPPD_PORT 164

#include <stdarg.h>
#include <syslog.h>
#include <sys/types.h>
#include <sys/socket.h>
//#include "asn1.h"
//#include "snmp_api.h"
//#include "snmp_impl.h"
//#include "snmp_client.h"
//#include "mib.h"
//#include "snmp.h"
#include "common.h"
#include "control.h"
#include "queue.h"


/* Typedefs */

typedef struct config_struct {
    int verbose;
    int debug;
    int nofork;
    char pid_file[32];
} config_t;


typedef struct object_struct {
    char objid[MAX_SNMPPD_OID_LEN];
    struct object_struct *next;
} object_t;

typedef struct target_struct {
    int sock;
    char *hostname;
    int port;
    object_t *objects;
    char *community;
//    int pollfd;
    pthread_t tid;
} target_t;

typedef struct poll_stats {
    long long polls;
    long long db_inserts;
    int wraps;
    int restarts;
    int no_resp;
    int out_of_range;
    int errors;
    int slow;
    double poll_time;
} stats_t;

#ifdef USE_THREADS
typedef struct work_queue {
    data_control control;
    queue work;
} work_queue;
/* I added a job number to the work node.  Normally, the work node
   would contain additional data that needed to be processed. */
typedef struct work_node {
    struct node *next;
    target_t *entry;
} wnode;
/* the cleanup queue holds stopped threads.  Before a thread
   terminates, it adds itself to this list.  Since the main thread is
   waiting for changes in this list, it will then wake up and clean up
   the newly terminated thread. */
typedef struct cleanup_queue {
    data_control control;
    queue cleanup;
} cleanup_queue;
/* I added a thread number (for debugging/instructional purposes) and
   a thread id to the cleanup node.  The cleanup node gets passed to
   the new thread on startup, and just before the thread stops, it
   attaches the cleanup node to the cleanup queue.  The main thread
   monitors the cleanup queue and is the one that performs the
   necessary cleanup. */
typedef struct cleanup_node {
    struct node *next;
    int threadnum;
    pthread_t tid;
} cnode;
#endif /* USE_THREADS */


/* Precasts */
extern int poll_target(target_t * head, char **buf, int buf_len);
extern void print_stats(stats_t stats);
extern void usage(char *executable);

extern void snmppd_snmp_init();
#ifdef USE_THREADS
extern void *thread_poll(void *);
extern pthread_mutex_t mem_mutex;
extern work_queue wq;
extern cleanup_queue cq;
#else
extern void child_main(target_t *entry);
#endif /* USE_THREADS */
extern void free_entry(target_t *entry);

extern void snmppd_log(int level, char *fmt, va_list args);
extern void log_error(char *fmt, ...);
extern void log_info(char *fmt, ...);
extern void log_debug(char *fmt, ...);

int write_pid(void);
/* Globals */
extern struct snmp_session session;
extern config_t config;


#endif				/* not _SNMPPD_H_ */
