#include "common.h"
#include "snmppd.h"
#include <signal.h>
#include <errno.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>

#ifdef MALLOC_DEBUG
#include "rmalloc/rmalloc.h"
#endif




void print_stats(stats_t stats)
{
    log_info
	("[Polls = %lld] [No Resp = %d] [SNMP Errors = %d]",
	 stats.polls, stats.no_resp, stats.errors);
}

void snmppd_log(int level, char *fmt, va_list args)
{
    char buffer[2048];
    vsnprintf(buffer, 2048, fmt, args);
    if (config.debug || config.nofork) {
	time_t now;
	char *time_str = NULL;
	char *p = NULL;
	now=time(NULL);
	time_str = ctime(&now);
	while ((p=strchr(time_str, '\n')) != NULL){
	    *p='\0';
	}
	fprintf(stdout, "[%s] %s", time_str, buffer);
	fputc('\n', stdout);
    } else {
	if (level >= LOG_INFO) {
	    syslog(level, buffer);
	}
    }
}

void log_error(char *fmt, ...)
{
    va_list args;

    va_start(args, fmt);
    snmppd_log(LOG_ERR, fmt, args);
    va_end(args);
}

void log_info(char *fmt, ...)
{
    va_list args;

    va_start(args, fmt);
    snmppd_log(LOG_INFO, fmt, args);
    va_end(args);
}

void log_debug(char *fmt, ...)
{
    va_list args;

    if (config.debug) {
	va_start(args, fmt);
	snmppd_log(LOG_DEBUG, fmt, args);
	va_end(args);
    }
}

int write_pid(void)
{
    FILE *f;
    char pid_str[32];
    unsigned int pid;
    mode_t prev_mask;

    if ((f = fopen(config.pid_file, "r")) != NULL) {
	/* pid_file already exists */
	if (fgets(pid_str, 30, f) != NULL) {
	    pid = atoi(pid_str);
	    if (pid != 0) {
		if ((kill(pid, 0) != 0) && (errno == ESRCH)) {
		    /* stale pid_file */
		    log_info("Stale pid file %s from process %u",
			     config.pid_file, pid);
		} else {
		    log_error("SNMPPD instance is already running (pid %u)",
			      pid);
		    return -1;
		}
	    }
	}
	fclose(f);
    }
    prev_mask = umask(022);
    if ((f = fopen(config.pid_file, "w")) != NULL) {
	fprintf(f, "%u\n", getpid());
	fclose(f);
	umask(prev_mask);
	return 0;
    } else {
	log_error("Can't open pid file %s for writing (%s)",
		  config.pid_file, strerror(errno));
	umask(prev_mask);
	return -1;
    }

}
