#include "common.h"
#ifdef USE_THREADS
#include <stdio.h>
#include <stdlib.h>
#include "threads.h"
#include "control.h"
#include "queue.h"
#include "dbug.h"
#include "snmppd.h"

#ifdef MALLOC_DEBUG
#include "rmalloc/rmalloc.h"
#endif


int numthreads;

void join_threads(void)
{
    cnode *curnode;
    if (config.debug) log_debug("joining threads...");
    while (numthreads) {
	pthread_mutex_lock(&cq.control.mutex);
	/* below, we sleep until there really is a new cleanup node.  This
	   takes care of any false wakeups... even if we break out of
	   pthread_cond_wait(), we don't make any assumptions that the
	   condition we were waiting for is true.  */
	while (cq.cleanup.head == NULL) {
	    pthread_cond_wait(&cq.control.cond, &cq.control.mutex);
	}
	/* at this point, we hold the mutex and there is an item in the
	   list that we need to process.  First, we remove the node from
	   the queue.  Then, we call pthread_join() on the tid stored in
	   the node.  When pthread_join() returns, we have cleaned up
	   after a thread.  Only then do we free() the node, decrement the
	   number of additional threads we need to wait for and repeat the
	   entire process, if necessary */
	curnode = (cnode *) queue_get(&cq.cleanup);
	pthread_mutex_unlock(&cq.control.mutex);
	pthread_join(curnode->tid, NULL);
	if (config.debug) log_debug("joined with thread %d", curnode->threadnum);
	free(curnode);
	numthreads--;
    }
}

int create_threads(void)
{
    int x;
    cnode *curnode;
    for (x = 0; x < NUM_WORKERS; x++) {
	curnode = malloc(sizeof(cnode));
	if (!curnode)
	    return 1;
	curnode->threadnum = x;
	if (pthread_create
	    (&curnode->tid, NULL, thread_poll, (void *) curnode))
	    return 1;
	if (config.debug) log_debug("created thread %d", x);
	numthreads++;
    }
    return 0;
}

void initialize_structs(void)
{
    numthreads = 0;
    if (control_init(&wq.control))
	dabort();
    queue_init(&wq.work);
    if (control_init(&cq.control)) {
	control_destroy(&wq.control);
	dabort();
    }
    queue_init(&wq.work);
    control_activate(&wq.control);
}

void cleanup_structs(void)
{
    control_destroy(&cq.control);
    control_destroy(&wq.control);
}
#endif /* USE_THREADS */
